unit Whexgrd;
{
   Hexgrid,
   Freeware, Copyright 1996, matthias@penthouse.boerde.de
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Controls,
  Graphics, Grids, Forms;

type

  TGetHexEvent = procedure (Sender: TObject; AAdr: Longint; var Value: byte) of object;
  TSetHexEvent = procedure (Sender: TObject; AAdr: Longint; const Value: byte) of object;

  TWHexGrid = class(TDrawGrid)
  private
    { Private-Deklarationen }
    FOldFontChange: TNotifyEvent;
    FHexColCount: Integer;
    FHexStartAdr: LongInt;
    FHexEndAdr:   LongInt;
    FGetHexEvent: TGetHexEvent;
    FSetHexEvent: TSetHexEvent;
    FAutoSizeCells: Boolean;
    FAutoSizeForm: Boolean;
    FRecalculateSize: Boolean;
    function CalcAdr(ACol, ARow: Integer): longint;
    procedure CalcColRow(adr: LongInt; var ACol, ARow: LongInt);
    procedure SetHexColCount( AHexColCount: Integer );
    procedure SetHexStartAdr( v: LongInt );
    procedure SetHexEndAdr( v: LongInt );
    procedure SetAutoSizeCells( b: boolean );
    procedure SetAutoSizeForm( b: boolean );
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure SetAdr( AAdr: LongInt );
    function  GetAdr: LongInt;
  protected
    { Protected-Deklarationen }
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
      AState: TGridDrawState); override;
    function  GetEditText(ACol, ARow: Longint): string; override;
    procedure SetEditText(ACol, ARow: Longint; const Value: string); override;
    function  GetEditLimit: Integer; override;
    function  GetAsciiColumn( ARow: LongInt ): string;
    procedure SetAsciiColumn( ARow: LongInt; s: string );
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    procedure CalculateSize;
    procedure UpdateSize;
  published
    { Published-Deklarationen }
    property OnSetHexValue: TSetHexEvent read FSetHexEvent write FSetHexEvent;
    property OnGetHexValue: TGetHexEvent read FGetHexEvent write FGetHexEvent;
    property HexColCount:  Integer read FHexColCount write SetHexColCount;
    property HexStartAdr:  LongInt read FHexStartAdr write SetHexStartAdr;
    property HexEndAdr:    LongInt read FHexEndAdr   write SetHexEndAdr;
    property AutoSizeCells:     Boolean read FAutoSizeCells    write SetAutoSizeCells;
    property AutoSizeForm: Boolean read FAutoSizeForm write SetAutoSizeForm;
    property Adr:          LongInt read GetAdr       write SetAdr;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TWHexGrid]);
end;

{tools}

function max( a,b: integer ):Integer;
begin
   if a>b then
      Result:=a
   else
      Result:=b;
end;

function GetHexDigitCount( v: longint ):integer;
var m:longint; i:integer;
begin
   if v>=0 then
   begin
     m:=1; i:=0;
     { $10 $100 $1000}
     while (v>=m) and (i<8) do
     begin
       m:=m*$10;
       i:=i+1;
     end;
     if i=0 then i:=1;
     Result:=i;
   end else
     Result:=8;
end;

function IntToHexA( Value, RefValue: Longint):string;
begin
     Result:=IntToHex(Value,GetHexDigitCount(RefValue));
end;

function CleanChar( c: char ): char;
begin
     if c<#32 then
        c:='.';
     Result:=c;
end;

function IsHexNumber( s: string): Boolean;
var i: integer;
begin
     Result:=True;
     for i:=1 to length(s) do
     begin
       if NOT (s[i] in ['0'..'9','a'..'f','A'..'F']) then
       begin
          Result:=False;
          break;
       end;
     end;
end;

constructor TWHexGrid.Create(AOwner: TComponent);
begin
    inherited Create( AOwner );
    ColCount:=18;
    Options:= Options + [goEditing];
    FHexColCount:=$10;
    FHexStartAdr:=0;
    FHexEndAdr:=$1f;
    FAutoSizeCells:=True;
    FAutoSizeForm:=False;
    FGetHexEvent:=nil;
    FSetHexEvent:=nil;
    FRecalculateSize:=True;
    Font.Pitch := fpFixed;
end;

procedure TWHexGrid.CalcColRow(adr: LongInt; var ACol, ARow: LongInt);
begin
      ACol := (adr-FHexStartAdr) mod HexColCount;
      ARow := (adr - ACol - FHexStartAdr) div HexColCount;
      ARow:=ARow+1;
      ACol:=ACol+1;
end;

function TWHexGrid.CalcAdr(ACol, ARow: Integer): longint;
begin
     Result := (ARow-1)*HexColCount + (ACol-1) + FHexStartAdr;
end;

procedure TWHexGrid.SetAsciiColumn( ARow: LongInt; s: string ) ;
var v: byte;
    i: integer;
    adr: LongInt;
begin
          for i:=1 to HexColCount do
          begin
              adr:=CalcAdr(i,ARow);
              if (adr>=FHexStartAdr) and (adr<=FHexEndAdr) then
              begin
                FGetHexEvent( self, adr, v);
                if (i<=Length(s)) and ( (v>=32) or (s[i]<>'.') ) then
                begin { write byte, ensure that ControlCharacters are not overwritten with '.'}
                  FSetHexEvent( self, adr, ord(s[i]) );
                end;
              end;
          end;
end;

function TWHexGrid.GetAsciiColumn( ARow: LongInt ): string;
var s: string;
    v: byte;
    i: integer;
    adr: LongInt;
begin
          s:='';
          for i:=1 to HexColCount do
          begin
              adr:=CalcAdr(i,ARow);
              if (adr>=FHexStartAdr) and (adr<=FHexEndAdr) then
              begin
                FGetHexEvent( self, adr, v);
                s:=s+CleanChar(chr(v));
              end;
          end;
          Result:=s;
end;

procedure TWHexGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
      AState: TGridDrawState);
var v: byte;
    i: integer;
    s: string;
    adr: LongInt;
begin
    CalculateSize;
    if ARow=0 then
    begin   {draw table header: top}
      if (ACol>0) and (ACol<=HexColCount) then
        Canvas.TextRect( ARect, ARect.Left+2, ARect.Top+2, IntToHex(ACol-1,2));
    end else if ACol=0 then
     begin  {draw table header: left}
      if ARow<>0 then
        Canvas.TextRect(ARect, ARect.Left+2, ARect.Top+2, IntToHexA(FHexStartAdr+(ARow-1)*HexColCount, FHexEndAdr))
     end else if Assigned(FGetHexEvent) then
      begin
        if ACol=HexColCount+1 then
        begin {draw ASCII}
          s:=GetAsciiColumn( ARow );
          Canvas.TextRect( ARect, ARect.Left+2, ARect.Top+2, s );
        end else begin {draw Cells}
          adr:=CalcAdr(ACol,ARow);
          if (adr>=FHexStartAdr) and (adr<=FHexEndAdr) then
          begin
               FGetHexEvent( self, adr, v);
               Canvas.TextRect( ARect, ARect.Left+2, ARect.Top+2, IntToHex(v,2) );
          end;
        end;
      end;
    inherited DrawCell(ACol, ARow, ARect, AState );
end;

function TWHexGrid.GetEditText(ACol, ARow: Longint): string;
var
   v: byte;
   adr: LongInt;
begin
     Result:='';
     if Assigned(FGetHexEvent) then
     begin
       if ACol<=HexColCount then
       begin         {HEX-Column}
         adr:=CalcAdr(ACol,ARow);
         if (adr>=FHexStartAdr) and (adr<=FHexEndAdr) then
         begin
            FGetHexEvent( self, adr, v);
            Result:=IntToHex( v, 2);
         end;
       end else begin { ASCII-Column }
         Result:=GetAsciiColumn( ARow );
       end;
     end;
end;

procedure TWHexGrid.SetEditText(ACol, ARow: Longint; const Value: string);
var
   adr: LongInt;
begin
  if Assigned(FSetHexEvent) and Assigned(FGetHexEvent) then
  begin
    if ACol<=HexColCount then
    begin
       if IsHexNumber(Value) then
       begin
         adr:=CalcAdr(ACol,ARow);
         if (adr>=FHexStartAdr) and (adr<=FHexEndAdr) then
         begin  {HEX Col}
           FSetHexEvent( self, adr, StrToInt('$0'+Value) );
           { force repaint of ASCII-Cell }
           Invalidate; {selectiv waere besser: CellRect( HexColCount+1, ARow);}
         end;
       end;
    end else begin { ASCII Col}
         SetAsciiColumn( ARow, Value );
         Invalidate;
    end;
  end;
end;

function TWHexGrid.GetEditLimit: Integer;
begin
     Result:=inherited GetEditLimit;
     if Col<=HexColCount then
        Result:=2
     else
        Result:=HexColCount;
end;

procedure TWHexGrid.SetHexColCount( AHexColCount: Integer );
begin
     if FHexColCount<>AHexColCount then
     begin
       FHexColCount:=AHexColCount;
       UpdateSize;
     end;
end;

procedure TWHexGrid.SetHexStartAdr( v: LongInt );
begin
     if FHexStartAdr<>v then
     begin
       FHexStartAdr:=v;
       UpdateSize;
     end;
end;

procedure TWHexGrid.SetHexEndAdr( v: LongInt );
begin
     if FHexEndAdr<>v then
     begin
       FHexEndAdr:=v;
       UpdateSize;
     end;
end;

procedure TWHexGrid.SetAutoSizeCells( b: boolean );
begin
    if FAutoSizeCells<>b then
    begin
      FAutoSizeCells:=b;
      UpdateSize;
    end;
end;

procedure TWHexGrid.SetAutoSizeForm( b: boolean );
begin
    if FAutoSizeForm<>b then
    begin
      FAutoSizeForm:=b;
      if b and (Align=alNone) then
         Align:=alTop;
      UpdateSize;
    end;
end;

procedure TWHexGrid.SetAdr( AAdr: LongInt );
var ACol, ARow: LongInt;
begin
  CalcColRow(AAdr, ACol, ARow);
  if (AAdr>=FHexStartAdr) and (AAdr<=FHexEndAdr) then
  begin
     Col:=ACol;
     Row:=ARow;
  end;
end;

function TWHexGrid.GetAdr: LongInt;
begin
  Result:=CalcAdr(Col, Row);
end;

procedure TWHexGrid.CMFontChanged(var Message: TMessage);
begin
  UpdateSize;
  inherited;
end;

procedure TWHexGrid.UpdateSize;
begin
     FRecalculateSize:=True;
     Invalidate;
end;

procedure TWHexGrid.CalculateSize;
var m: TTextMetric;
var i,w: integer;
begin
  if FReCalculateSize then
  begin
     ColCount:=FHexColCount+2;
     if FHexEndAdr>=FHexStartAdr then
     begin
        RowCount:=2+ (FHexEndAdr-FHexStartAdr) div FHexColCount;
     end;

     if FAutoSizeCells then
     begin
       {
       GetTextMetrics( Canvas.Handle, m );
       w:=m.tmAveCharWidth;
       }
       w:=Canvas.TextWidth( 'D' );
       DefaultColWidth:=4+ w*2;
       ColWidths[0]:=4+ w* max(GetHexDigitCount(FHexStartAdr),GetHexDigitCount(FHexEndAdr));
       ColWidths[HexColCount+1]:=4+ w*FHexColCount;
       DefaultRowHeight:=4+Canvas.TextHeight( IntToHex( $DD, 2 ) );
       {update size of Form}
       if FAutoSizeForm then
       begin
          w:=0;
          for i:=0 to HexColCount+1 do
              w:=w+ColWidths[i];
          w:=w+GridLineWidth*(HexColCount+3)+GetSystemMetrics(SM_CXVSCROLL);
          if Owner is TForm then
            TForm(Owner).ClientWidth:=w;
       end;
     end;
     Invalidate;
     FRecalculateSize:=False;
   end;
end;

end.

