unit Umain;

{
  IIC-Bus Universal

  Freeware, Copyright 1996, Matthias Weingart;
  comments, bug reports to matthias@penthouse.boerde.de

  1.2.18: NVM-routines moved to UniI2c
          faster NVM-Access: no hourglass cursor on NVM access
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Menus, UniI2c, IniFiles, DStamp, Buttons,
  Spin, UDump,USetBus, UAbout;

type
  TForm1 = class(TForm)
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    SendMemo: TMemo;
    Panel2: TPanel;
    ReceiveMemo: TMemo;
    DateStamp1: TDateStamp;
    Panel3: TPanel;
    Label1: TLabel;
    Panel4: TPanel;
    SendBtn: TButton;
    SendRepeatBtn: TButton;
    I2CListBtn: TButton;
    I2CCancelBtn: TButton;
    SpinButton1: TSpinButton;
    SendAllCB: TCheckBox;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Loadsendfile1: TMenuItem;
    Savesendfile1: TMenuItem;
    Savereceivefile1: TMenuItem;
    Extra1: TMenuItem;
    Dump1: TMenuItem;
    settings1: TMenuItem;
    Exit1: TMenuItem;
    Label3: TLabel;
    PortBox: TComboBox;
    Label2: TLabel;
    BrakeBox: TComboBox;
    Label4: TLabel;
    BaseAdr0Edit: TEdit;
    Label5: TLabel;
    BaseAdr2Edit: TEdit;
    SetBtn: TButton;
    About1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure SendBtnClick(Sender: TObject);
    procedure I2CCancelBtnClick(Sender: TObject);
    procedure I2CListBtnClick(Sender: TObject);
    procedure PortBoxChange(Sender: TObject);
    procedure PortBoxKeyPress(Sender: TObject; var Key: Char);
    procedure SendMemoKeyPress(Sender: TObject; var Key: Char);
    procedure BrakeBoxClick(Sender: TObject);
    procedure BrakeBoxKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpinButton1UpClick(Sender: TObject);
    procedure SpinButton1DownClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Loadsendfile1Click(Sender: TObject);
    procedure Savesendfile1Click(Sender: TObject);
    procedure Savereceivefile1Click(Sender: TObject);
    procedure Dump1Click(Sender: TObject);
    procedure settings1Click(Sender: TObject);
    procedure SetBtnClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    { Private-Deklarationen }
    I2C: TPI2C;
    Cancel: boolean;
    Ini: TIniFile;
    FName: string;
    procedure PortChange;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var p: TLptType;
    s: string;
    v,i:integer;
begin
   AboutBox := TAboutBox.Create( Self );
   AboutBox.Parent := nil;
   AboutBox.Button1.Enabled:=False;
   AboutBox.Show;
   Application.ProcessMessages;
   Application.ProcessMessages;
   try
     Ini := TIniFile.Create( ChangeFileExt(Application.ExeName,'.ini' ));
     I2C:= TPI2C.Create( self );
     I2C.ErrorLabel := Label1;

     BusSettingsDlg := TBusSettingsDlg.Create( Self );
     BusSettingsDlg.Parent := nil;
     BusSettingsDlg.Ini:=Ini;
     BusSettingsDlg.InitProfileCb;

     for p:=tlLPT1 to tlLPT4 do
     begin
          I2C.Port:=p;
          case p of
            tlLPT1: s:='LPT1 ';
            tlLPT2: s:='LPT2 ';
            tlLPT3: s:='LPT3 ';
            tlLPT4: s:='LPT4 ';
          end;
          if I2c.PortAdr<>0 then
             PortBox.Items.Add( s+ IntToHex(I2c.PortAdr,4));
     end;
     PortBox.Text:=Ini.ReadString( 'Settings', 'Port', 'LPT1' );
     PortChange;

     s:=Ini.ReadString( 'Settings', 'Brake', 'Auto:' );
     if Pos('uto',LowerCase(s))=0 then
     begin
        Val(s,v,i);
        if i=0 then
        begin
           I2C.DelayCount:=v;
           BrakeBox.Text := IntToStr(v);
        end
        else
          BrakeBox.Text := 'Auto: '+IntToStr(I2C.DelayCount);
     end
     else
        BrakeBox.Text := 'Auto: '+IntToStr(I2C.DelayCount);

     BusSettingsDlg.SetProfile( I2C );
     I2C.CheckBus; {set i2c-label}
     BaseAdr0Edit.Text := IntToHex( I2C.PortValue0,2 );
     BaseAdr2Edit.Text := IntToHex( I2C.PortValue2,2 );

     SendAllCB.Checked:=Ini.ReadBool( 'Settings', 'SendAll', False );
     if ParamCount>0 then
        SendMemo.Lines.LoadFromFile( ParamStr(1) );
     I2C.BeepCount:=Ini.ReadInteger( 'Settings', 'BeepCount', 0 );
   finally
     AboutBox.Hide;
     AboutBox.Button1.Enabled:=True;
   end;
end;


procedure InterpretString( s: string; var count: integer; var o: array of integer);
var pos: integer;
    v:   string;
    RepeatStart,Receive: Boolean;
begin
  s:=s+' '; {damit er auch letztes byte nimmt}
  pos := 1;
  v:=''; count:=0; RepeatStart:=False; Receive:=False;
  while pos<=length(s) do
  begin
    case s[pos] of
       ' ',';',':',',','.':
              begin
              if Receive then
              begin
                 o[count]:=CReceive or $ff;
                 Receive:=False;
                 v:=''; count:=count+1;
              end
              else if length(v)>0 then
              begin
                o[count]:=StrToInt('$0'+v) and $ff;
                if RepeatStart then
                   o[count]:=o[count] or CStartbus;
                RepeatStart:=False;
                v:=''; count:=count+1;
              end;
              if s[pos]=';' then break;
              if s[pos] in [':',',','.'] then
              begin
                 RepeatStart:=True;
                 if count>0 then
                   o[count-1]:=o[count-1] or CStopbus;
              end;
            end;
       '0'..'9','A'..'F','a'..'f': v:=v+s[pos];
       's','S': RepeatStart:=True;
       'p','P': if count>0 then
                   o[count-1]:=o[count-1] or CStopbus;
       'x','X','r','R': Receive:=True;
       else
       begin
           Application.MessageBox('Syntaxerror','Syntax?',MB_OK);
           count:=0;
           break;
       end;
    end;
    pos := pos + 1;
  end;
  o[0]:=o[0] or CStartbus;
  if count>0 then
     o[count-1]:=o[count-1] or CStopbus;
end;

procedure TForm1.SendBtnClick(Sender: TObject);
var o,i: array[0..256] of integer;
    b: Boolean;
    s: string; k: integer;
var
    count, linestart, lineend, line: integer;
    outline, startline: integer;
begin
  SendBtn.Enabled:=False;
  SendRepeatBtn.Enabled:=False;
  if SendAllCB.Checked then
  begin
    linestart:=0;
    {Zeile, des Endes der Markierung}
    lineend:=SendMemo.Lines.Count-1;
  end else begin
    {Zeile, die Cursor hat}
    linestart:=SendMessage( SendMemo.Handle, EM_LINEFROMCHAR, SendMemo.SelStart, 0);
    {Zeile, des Endes der Markierung}
    lineend:=SendMessage( SendMemo.Handle, EM_LINEFROMCHAR, SendMemo.SelStart+SendMemo.SelLength, 0);
  end;
  Cancel:=False;
  startline:=ReceiveMemo.Lines.Count;
  repeat
    outline:=startline;
    for line:=linestart to lineend do
    begin
     InterpretString( SendMemo.Lines[line], count, o);
     if count>0 then
     begin
       I2C.Universal( count, o, i);
       s:='';
       for k:=0 to count-1 do
       begin
         if (i[k] AND CStartbus) = CStartbus then
            s:=s+'s';
         if (i[k] AND CReceive) = CReceive then
            s:=s+'r';
         s:=s+IntToHex( i[k] AND $ff, 2);
         if (i[k] AND CAck) = CAck then
            s:=s+'+'
         else
            s:=s+'-';
         if (i[k] AND CStopbus) = CStopbus then
            s:=s+'p';
         s:=s+' ';
       end;
       try
        if sender = SendRepeatBtn then
         if outline>=ReceiveMemo.Lines.Count then
            ReceiveMemo.Lines.Add( s )
         else
            ReceiveMemo.Lines[outline]:=s
        else
          ReceiveMemo.Lines.Add( s );
        except
          ReceiveMemo.Clear;
        end;
       outline:=outline+1;
     end;
     Application.ProcessMessages;
    end;
  until (sender <> SendRepeatBtn) or Cancel;
  SendBtn.Enabled:=True;
  SendRepeatBtn.Enabled:=True;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     Ini.WriteBool( 'Settings', 'SendAll', SendAllCB.Checked );
end;

procedure TForm1.I2CCancelBtnClick(Sender: TObject);
begin
     I2C.Cancel;
     Cancel:=True;
end;

procedure TForm1.I2CListBtnClick(Sender: TObject);
var i: integer;
begin
     i:=0;
     repeat
       if I2C.Available( i ) then
       begin
         ReceiveMemo.Lines.Add( 'Dev: '+IntToHex(i,2) );
       end;
       i:=i+2;
     until i>=$ff;
end;

procedure TForm1.PortChange;
begin
     if Pos('LPT1',PortBox.Text)>0 then I2C.Port:=tlLPT1
      else if Pos('LPT2',PortBox.Text)>0 then I2C.Port:=tlLPT2
       else if Pos('LPT3',PortBox.Text)>0 then I2C.Port:=tlLPT3
        else if Pos('LPT4',PortBox.Text)>0 then I2C.Port:=tlLPT4
         else I2C.PortAdr:=StrToInt( '$0'+ PortBox.Text );
     I2C.CheckBus; {set i2c-label}
end;

procedure TForm1.PortBoxChange(Sender: TObject);
begin
   PortChange;
   with Ini do
   begin
     if Pos('LPT1',PortBox.Text)>0 then Ini.WriteString( 'Settings', 'Port', 'LPT1' )
      else if Pos('LPT2',PortBox.Text)>0 then Ini.WriteString( 'Settings', 'Port', 'LPT2' )
       else if Pos('LPT3',PortBox.Text)>0 then Ini.WriteString( 'Settings', 'Port', 'LPT3' )
        else if Pos('LPT4',PortBox.Text)>0 then Ini.WriteString( 'Settings', 'Port', 'LPT4' )
         else Ini.WriteString( 'Settings', 'Port', PortBox.Text );
   end;
end;

procedure TForm1.PortBoxKeyPress(Sender: TObject; var Key: Char);
begin
     if Key=#13 then SendBtn.SetFocus;
end;

procedure TForm1.SendMemoKeyPress(Sender: TObject; var Key: Char);
begin
     if key=#13 then SendBtn.Click;
end;

procedure TForm1.BrakeBoxClick(Sender: TObject);
var s: string;
begin
   if Pos('auto',LowerCase(BrakeBox.Text))>0 then
   begin
      Screen.Cursor:=crHourglass;
      I2C.InitDelay;
      s := 'Auto: '+IntToStr(I2C.DelayCount);
      BrakeBox.Text:=s;
      Ini.WriteString( 'Settings', 'Brake', 'Auto:' );
      Screen.Cursor:=crDefault;
   end
   else
   begin
      I2C.DelayCount:=StrToInt(BrakeBox.Text);
      Ini.WriteString( 'Settings', 'Brake', IntToStr(I2C.DelayCount) );
   end;
end;

procedure TForm1.BrakeBoxKeyPress(Sender: TObject; var Key: Char);
begin
     if Key=#13 then SendBtn.SetFocus;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
     Label1.Hint := Label1.Hint + DateStamp1.Version + '.' + IntToStr(DateStamp1.VersionCount);
end;

procedure SelectWord( Memo: TMemo );
var i:integer;
    s:string;
    c:char;
begin
     {vorhergehendes Leerzeichen finden}
     i:=0;
     repeat
          Memo.SelStart := Memo.SelStart -1;
          Memo.SelLength:=1;
          i:=i+1;
          c:=Memo.SelText[1];
     until (not (c in['0'..'9','a'..'f','A'..'F'])) or (i>3) or (Memo.SelStart=0);
     if Memo.SelStart<>0 then Memo.SelStart := Memo.SelStart + 1;
     {folgendes Leerzeichen finden}
     i:=0;
     repeat
          Memo.SelLength := Memo.SelLength+1;
          i:=i+1;
          c:=Memo.SelText[Memo.SelLength];
     until (not (c in['0'..'9','a'..'f','A'..'F'])) or (i>3);
     if i<4 then Memo.SelLength := Memo.SelLength - 1;
end;

procedure TForm1.SpinButton1UpClick(Sender: TObject);
var i:integer;
begin
    SelectWord(SendMemo);
    i:=StrToInt('$0'+ SendMemo.SelText);
    i:=(i+1) and $ff;
    SendMemo.SelText := IntToHex(i, 2);
end;

procedure TForm1.SpinButton1DownClick(Sender: TObject);
var i:integer;
begin
    SelectWord(SendMemo);
    i:=StrToInt('$0'+ SendMemo.SelText);
    i:=(i-1) and $ff;
    SendMemo.SelText := IntToHex(i, 2);
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
     I2C.Cancel;
     Cancel:=True;
     close;
end;

procedure TForm1.Loadsendfile1Click(Sender: TObject);
begin
     OpenDialog1.FileName:=FName;
     if OpenDialog1.Execute then
     begin
        FName:=OpenDialog1.FileName;
        Caption:=Application.Title+' - '+FName;
        SendMemo.Lines.LoadFromFile( OpenDialog1.FileName );
     end;
end;

procedure TForm1.Savesendfile1Click(Sender: TObject);
begin
     SaveDialog1.FileName:=FName;
     if SaveDialog1.Execute then
     begin
        SendMemo.Lines.SaveToFile( SaveDialog1.FileName );
        FName:=SaveDialog1.FileName;
        Caption:=Application.Title+' - '+FName;
     end;
end;

procedure TForm1.Savereceivefile1Click(Sender: TObject);
begin
     if SaveDialog1.Execute then
        ReceiveMemo.Lines.SaveToFile( SaveDialog1.FileName );
end;

procedure TForm1.Dump1Click(Sender: TObject);
begin
     DumpForm.I2C:=I2C;
     DumpForm.Show;
end;

procedure TForm1.settings1Click(Sender: TObject);
begin
   with BusSettingsDlg do
   begin
     GetProfile( I2C );
     if BusSettingsDlg.ShowModal=mrOk then
     begin
       SetProfile( I2C );
       Save;
     end;
   end;
end;

procedure TForm1.SetBtnClick(Sender: TObject);
begin
     I2C.PortValue0:=StrToInt( '$0'+BaseAdr0Edit.Text );
     I2C.PortValue2:=StrToInt( '$0'+BaseAdr2Edit.Text );
     BaseAdr0Edit.Text := IntToHex( I2C.PortValue0,2 );
     BaseAdr2Edit.Text := IntToHex( I2C.PortValue2,2 );
end;

procedure TForm1.About1Click(Sender: TObject);
begin
     AboutBox.Version.Caption:=DateStamp1.Version + '.' + IntToStr(DateStamp1.VersionCount);
     AboutBox.LastModified.Caption:=DateStamp1.Date + '  ' + DateStamp1.Time;
     AboutBox.ShowModal;
end;

end.
